@@ -65,6 +65,9 @@
 #else
 #include <asm/uaccess.h>
 #endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 11, 0)
+#include <linux/sched/signal.h>
+#endif
 #include <net/sock.h>
 #include <net/af_unix.h>
 #include <net/ip.h>
@@ -1968,7 +1971,7 @@ extern struct ccsecurity_exports ccsecurity_exports;
  */
 struct ccs_security {
 	struct list_head list;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29) || LINUX_VERSION_CODE >= KERNEL_VERSION(4, 12, 0)
 	const struct task_struct *task;
 #else
 	struct pid *pid; /* Maybe NULL. */
