@@ -18,6 +18,29 @@
 
 #include "probe.h"
 
+static void *__init check_function_address(void *ptr, char *symbol)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+	static char buf[KSYM_SYMBOL_LEN];
+	const int len = strlen(symbol);
+#endif
+
+	if (!ptr) {
+		printk(KERN_ERR "Can't resolve %s().\n", symbol);
+		return NULL;
+	}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+	snprintf(buf, sizeof(buf), "%pS", ptr);
+	if (strncmp(buf, symbol, len) || strncmp(buf + len, "+0x0/", 5)) {
+		printk(KERN_ERR "Guessed %s is %s\n", symbol, buf);
+		return NULL;
+	}
+#else
+	printk(KERN_INFO "%s=%p\n", symbol, ptr);
+#endif
+	return ptr;
+}
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
 
 /**
@@ -41,6 +64,7 @@ static int __init probe_kernel_read(struct file *file, unsigned long offset,
 	mm_segment_t old_fs;
 	unsigned long pos = file->f_pos;
 	int result;
+
 	file->f_pos = offset;
 	old_fs = get_fs();
 	set_fs(get_ds());
@@ -52,6 +76,7 @@ static int __init probe_kernel_read(struct file *file, unsigned long offset,
 	return kernel_read(file, offset, addr, count);
 #else
 	loff_t pos = offset;
+
 	return kernel_read(file, addr, count, &pos);
 #endif
 }
@@ -68,6 +93,7 @@ static void *__init probe_find_symbol(const char *keyline)
 	struct file *file = NULL;
 	char *buf;
 	unsigned long entry = 0;
+
 	{
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 		struct file_system_type *fstype = get_fs_type("proc");
@@ -81,6 +107,7 @@ static void *__init probe_find_symbol(const char *keyline)
 #endif
 		struct dentry *root;
 		struct dentry *dentry;
+
 		/*
 		 * We embed put_filesystem() here because it is not exported.
 		 */
@@ -108,6 +135,7 @@ static void *__init probe_find_symbol(const char *keyline)
 		else {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
 			struct path path = { .mnt = mnt, .dentry = dentry };
+
 			file = dentry_open(&path, O_RDONLY, current_cred());
 #else
 			file = dentry_open(dentry, mnt, O_RDONLY
@@ -124,9 +152,11 @@ static void *__init probe_find_symbol(const char *keyline)
 	if (buf) {
 		int len;
 		int offset = 0;
+
 		while ((len = probe_kernel_read(file, offset, buf,
 						PAGE_SIZE - 1)) > 0) {
 			char *cp;
+
 			buf[len] = '\0';
 			cp = strrchr(buf, '\n');
 			if (!cp)
@@ -170,6 +200,7 @@ void probe_security_bprm_committed_creds(struct linux_binprm *bprm)
 {
 	do {
 		struct security_hook_list *p;
+
 		list_for_each_entry(p, &probe_dummy_security_hook_heads.
 				    bprm_committed_creds, list)
 			p->hook.bprm_committed_creds(bprm);
@@ -195,6 +226,7 @@ static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
 	for (i = 0; i < 32; ip++, i++) {
 		unsigned long tmp;
 		unsigned long offset;
+
 		/*
 		 * Find
 		 *   adrp Xd, #imm21
@@ -243,6 +275,7 @@ static void * __init probe_security_hook_heads_on_arm(unsigned int *base)
 	const unsigned long addr = (unsigned long) &probe_dummy_security_hook_heads;
 	const unsigned long offset = (unsigned long) &probe_dummy_security_hook_heads.bprm_committed_creds - addr;
 	unsigned int *ip = (unsigned int *) probe_security_bprm_committed_creds;
+
 	for (i = 0; i < 32; ip++, i++) {
 		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
 			continue;
@@ -291,6 +324,7 @@ static void * __init probe_security_ops_on_arm(unsigned int *base)
 	int i;
 	const unsigned long addr = (unsigned long) &probe_dummy_security_ops;
 	unsigned int *ip = (unsigned int *) probe_security_file_alloc;
+
 	for (i = 0; i < 32; ip++, i++) {
 		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
 			continue;
@@ -340,8 +374,10 @@ static void * __init probe_find_vfsmount_lock_on_arm(unsigned int *ip,
 						     unsigned int *base)
 {
 	int i;
+
 	for (i = 0; i < 32; ip++, i++) {
 		static unsigned int *ip4ret;
+
 		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
 			continue;
 		ip = base + i;
@@ -372,6 +408,7 @@ static void * __init probe_find_variable(void *function, unsigned long addr,
 	int i;
 	u8 *base;
 	u8 *cp = function;
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
 	if (*symbol == ' ')
 		base = probe_find_symbol(symbol);
@@ -408,6 +445,7 @@ static void * __init probe_find_variable(void *function, unsigned long addr,
 	for (i = 0; i < 128; i++) {
 		if ((unsigned long) (cp + (*(int *) cp) * 2 - 2) == addr) {
 			static void *cp4ret;
+
 			cp = base + i;
 			cp += (*(int *) cp) * 2 - 2;
 			cp4ret = cp;
@@ -420,6 +458,7 @@ static void * __init probe_find_variable(void *function, unsigned long addr,
 	for (i = 0; i < 128; i++) {
 		if ((unsigned long) (cp + sizeof(int) + *(int *) cp) == addr) {
 			static void *cp4ret;
+
 			cp = base + i;
 			cp += sizeof(int) + *(int *) cp;
 			cp4ret = cp;
@@ -431,6 +470,7 @@ static void * __init probe_find_variable(void *function, unsigned long addr,
 	for (i = 0; i < 128; i++) {
 		if ((unsigned long) (long) (*(int *) cp) == addr) {
 			static void *cp4ret;
+
 			cp = base + i;
 			cp = (void *) (long) (*(int *) cp);
 			cp4ret = cp;
@@ -453,6 +493,14 @@ struct security_hook_heads * __init probe_security_hook_heads(void)
 	const unsigned int offset = offsetof(struct security_hook_heads,
 					     bprm_committed_creds);
 	void *cp;
+	struct security_hook_heads *shh;
+	struct security_hook_list *entry;
+	void *cap = probe_find_symbol(" cap_bprm_set_creds\n");
+
+	/* Get location of cap_bprm_set_creds(). */
+	cap = check_function_address(cap, "cap_bprm_set_creds");
+	if (!cap)
+		return NULL;
 	/* Guess "struct security_hook_heads security_hook_heads;". */
 	cp = probe_find_variable(probe_security_bprm_committed_creds,
 				 ((unsigned long)
@@ -464,9 +512,13 @@ struct security_hook_heads * __init probe_security_hook_heads(void)
 		return NULL;
 	}
 	/* This should be "struct security_hook_heads security_hook_heads;". */
-	cp = ((void *) (*(unsigned long *) cp)) - offset;
-	printk(KERN_INFO "security_hook_heads=%p\n", cp);
-	return cp;
+	shh = ((void *) (*(unsigned long *) cp)) - offset;
+	list_for_each_entry(entry, &shh->bprm_set_creds, list)
+		if (entry->hook.bprm_set_creds == cap)
+			return shh;
+	printk(KERN_ERR "Guessed security_hook_heads is 0x%lx\n",
+	       (unsigned long) shh);
+	return NULL;
 }
 
 #else
@@ -482,6 +534,7 @@ struct security_operations * __init probe_security_ops(void)
 	struct security_operations *ops;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
 	void *cp;
+
 	/* Guess "struct security_operations *security_ops;". */
 	cp = probe_find_variable(probe_security_file_alloc, (unsigned long)
 				 &probe_dummy_security_ops,
@@ -521,17 +574,13 @@ struct security_operations * __init probe_security_ops(void)
 void * __init probe_find_task_by_vpid(void)
 {
 	void *ptr;
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
 	ptr = probe_find_symbol(" find_task_by_vpid\n");
 #else
 	ptr = __symbol_get("find_task_by_vpid");
 #endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve find_task_by_vpid().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "find_task_by_vpid=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "find_task_by_vpid");
 }
 
 /**
@@ -542,17 +591,13 @@ void * __init probe_find_task_by_vpid(void)
 void * __init probe_find_task_by_pid_ns(void)
 {
 	void *ptr;
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
 	ptr = probe_find_symbol(" find_task_by_pid_ns\n");
 #else
 	ptr = __symbol_get("find_task_by_pid_ns");
 #endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve find_task_by_pid_ns().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "find_task_by_pid_ns=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "find_task_by_pid_ns");
 }
 
 #endif
@@ -581,6 +626,7 @@ static int probe_hash_mask, probe_hash_bits;
 static inline unsigned long hash(struct vfsmount *mnt, struct dentry *dentry)
 {
 	unsigned long tmp = ((unsigned long) mnt / L1_CACHE_BYTES);
+
 	tmp += ((unsigned long) dentry / L1_CACHE_BYTES);
 	tmp = tmp + (tmp >> probe_hash_bits);
 	return tmp & probe_hash_mask;
@@ -626,6 +672,7 @@ void * __init probe_vfsmount_lock(void)
 {
 	void *cp;
 	spinlock_t *ptr;
+
 	/* Guess "spinlock_t vfsmount_lock;". */
 	cp = probe_find_variable(probe_lookup_mnt, (unsigned long)
 				 &probe_dummy_vfsmount_lock, " lookup_mnt\n");
@@ -660,6 +707,7 @@ static int probe_follow_up(struct vfsmount **mnt, struct dentry **dentry)
 {
 	struct vfsmount *parent;
 	struct dentry *mountpoint;
+
 	spin_lock(&probe_dummy_vfsmount_lock);
 	parent = (*mnt)->mnt_parent;
 	if (parent == *mnt) {
@@ -685,6 +733,7 @@ void * __init probe_vfsmount_lock(void)
 {
 	void *cp;
 	spinlock_t *ptr;
+
 	/* Guess "spinlock_t vfsmount_lock;". */
 	cp = probe_find_variable(probe_follow_up, (unsigned long)
 				 &probe_dummy_vfsmount_lock, "follow_up");
@@ -730,6 +779,7 @@ void * __init probe_vfsmount_lock(void)
 {
 	void *cp;
 	spinlock_t *ptr;
+
 	/* Guess "spinlock_t vfsmount_lock;". */
 	cp = probe_find_variable(probe_mnt_pin, (unsigned long)
 				 &probe_dummy_vfsmount_lock, "mnt_pin");
@@ -781,12 +831,8 @@ void * __init probe_vfsmount_lock(void)
 void * __init probe___d_path(void)
 {
 	void *ptr = probe_find_symbol(" __d_path\n");
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve __d_path().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "__d_path=%p\n", ptr);
-	return ptr;
+
+	return check_function_address(ptr, "__d_path");
 }
 
 #endif
@@ -801,12 +847,8 @@ void * __init probe___d_path(void)
 void * __init probe_d_absolute_path(void)
 {
 	void *ptr = probe_find_symbol(" d_absolute_path\n");
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve d_absolute_path().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "d_absolute_path=%p\n", ptr);
-	return ptr;
+
+	return check_function_address(ptr, "d_absolute_path");
 }
 
 #endif
